import { React, useState, useEffect } from 'react'
import { useNavigate } from 'react-router-dom'
import FilmItem from './components/FilmItem'
import ContentBlock from './components/ContentBlock'
import ModalEdit from './components/ModalEdit'
import Banner from './components/Banner'
import '../styles/banner.css'
import CinemaDto from './models/CinemaDto'
import Service from './services/Service'

export default function Films() {
    const navigate = useNavigate();
    const fileReader = new FileReader()
    const [items, setItems] = useState([]);
    const [modalTable, setModalTable] = useState(false);
    const [selectedImage, setSelectedImage] = useState();
    // для инпутов
    const [filmName, setFilmName] = useState('');

    const [filmNameEdit, setFilmNameEdit] = useState('');
    // хук для запоминания индекса элемента, вызвавшего модальное окно
    const [currEditItem, setCurrEditItem] = useState(0);

    // фильмы, страны, жанры
    useEffect(() => {
        Service.readAll('cinema')
            .then(data => setItems(data));
    }, []);

    function handleDeleteFilm(id) {
        console.info('Try to remove item');
        Service.delete(`cinema/${id}`)
            .then(() => {
                setItems(items.filter(elem => elem.id !== id))
                console.log("Removed")
            });
    };

    function handleEditFilm(id) {
        console.info(`Start edit script`);

        Service.read(`cinema/${id}`)
            .then(function (data) {
                setFilmNameEdit(data.name);
                setCurrEditItem(data.id);
                setModalTable(true)
                console.info('End edit script');
            })
    };

    // принимаем событие от кнопки "добавить"
    const handleSubmitCreate = (e) => {
        e.preventDefault(); // страница перестает перезагружаться
        const itemObject = new CinemaDto(selectedImage, filmName);
        console.info('Try to add item');

        Service.create('cinema', itemObject)
            .then((data) => {
                setItems([...items, data]);

                setFilmName('');
                console.info('Added');
            })
            .catch(function (error) {
                console.error('Error:', error);
                throw "Can't add item";
            });
    };

    // принимаем событие от кнопки "сохранить изменения"
    const handleSubmitEdit = (e, id) => {
        console.info('Start synchronize edit');
        e.preventDefault(); // страница перестает перезагружаться
        const itemObject = new CinemaDto(selectedImage, filmNameEdit, id);

        Service.update("cinema/" + id, itemObject)
            .then((data) => {
                setItems(
                    items.map(item =>
                        item.id === id ? {
                            ...item,
                            image: data.image,
                            name: data.name,
                        } : item)
                )
                console.info('End synchronize edit');
                setModalTable(false)
            })
            .catch((error) => {
                console.error('Error:', error);
            });

    };

    fileReader.onloadend = () => (
        setSelectedImage(fileReader.result)
    )
    function changePicture(e) {
        e.preventDefault();
        const file = e.target.files[0]
        fileReader.readAsDataURL(file)
    }

    function handleAddFilmToWillSee(data) {
        const elem = CinemaDto.createFrom(data)
        Service.create('willSee/',  data)
    }

    // форма для добавления данных
    const Form = (
        <form className="row g-3 fs-4 description fw-bold needs-validation-content" id="frm-items" onSubmit={handleSubmitCreate}>
            <div className="col-md-3">
                <label className="form-label" htmlFor="filmPicture">Изображение</label>
                <input required className="form-control" name="filmPicture" id="filmPicture"
                    type="file"
                    onChange={changePicture} />
            </div>
            <div className="col">
                <label className="form-label" htmlFor="filmName">Название</label>
                <input required className="form-control" name="filmName" id="filmName" type="text" value={filmName} onChange={e => setFilmName(e.target.value)} placeholder="Введите название" />
            </div>
            <div className="form-check mx-2">
                <input required className="form-check-input" name="film16" id="film16" type="checkbox" />
                <label className="form-check-label" htmlFor="film16">Мне уже 16 лет</label>
                <div className="invalid-feedback">Подтвердите, что вам уже есть 16 лет</div>
            </div>
            <div className="text-start">
                <button className="willSee p-1 border border-0 rounded text-white fw-bold fs-4" id="btn-add-item" type="submit">Добавить</button>
            </div>
        </form>
    )

    const Content = (
        <div>
            {Form}
            <hr className="border border-0 bg-black" />
            <table className="table" id="tbl-items">
                <tbody>
                    {items.map((item) =>
                        <FilmItem
                            item={item}
                            key={item.id}
                            removeFunc={handleDeleteFilm}
                            editFunc={handleEditFilm}
                            openFilmPageFunc={(index) => navigate(`/films/${index}`)}
                        />
                    )}
                </tbody>
            </table>
            <ModalEdit visible={modalTable} setVisible={setModalTable}>
                <form className="g-3 fs-4 description fw-bold container" id="frm-items-edit" onSubmit={(e) => handleSubmitEdit(e, currEditItem)}>
                    <div className="row">
                        <label className="form-label" htmlFor="filmPictureEdit">Изображение</label>
                        <input required className="form-control" name="filmPictureEdit" id="filmPictureEdit"
                            type="file"
                            onChange={changePicture} />
                    </div>
                    <div className="row">
                        <label className="form-label" htmlFor="filmNameEdit">Название</label>
                        <input value={filmNameEdit} onChange={e => setFilmNameEdit(e.target.value)} className="form-control" name='filmNameEdit' id="filmNameEdit" type="text" placeholder="Введите название" required />
                    </div>
                    <div className="text-center mt-3">
                        <button className="btn btn-primary mx-1" type="submit" id="buttonSaveChanges">Сохранить изменения</button>
                        <button className="btn btn-secondary mx-1" type="button" data-bs-dismiss="modal" onClick={() => setModalTable(false)}>Отмена</button>
                    </div>
                </form>
            </ModalEdit>
        </div>
    );

    return (
        <div>
            <Banner />
            <ContentBlock valueBlock={Content} title='Фильмы' />
        </div>
    )
}
