import { React, useState, useEffect } from 'react'
import ContentBlock from './components/ContentBlock'
import Service from './services/Service';
import ModalEdit from './components/ModalEdit';
import OrderItem from './components/OrderItem';
import OrderSessionItem from './components/OrderSessionItem';

export default function Orders() {
    const [users, setUsers] = useState([]);
    const [modalTable, setModalTable] = useState(false);
    // хук для запоминания индекса элемента, вызвавшего модальное окно
    const [currEditItem, setCurrEditItem] = useState(0);
    // для выпадающих значений
    const [customerName, setCustomerName] = useState('');
    const [customer, setCustomer] = useState([]);
    const [sessionName, setSessionName] = useState('');
    const [count, setCount] = useState(1);
    const [session, setSession] = useState([]);
    const [orderSessions, setOrderSessions] = useState([]);

    useEffect(() => {
        getAll('customer').then((data) => setCustomer(data))
        getAll('session').then((data) => setSession(data))
        getAll('order').then((data) => setUsers(data))
    }, [])

    async function getAll(elem) {
        const requestUrl = `http://localhost:8080/${elem}`
        const response = await fetch(requestUrl)
        const result = await response.json()
        return result
    }

    function handleSubmit(e) {
        e.preventDefault();
        if (customer.length <= 0) {
            throw 'Form not submit'
        }
        handleSubmitCreate(e)
        console.log('Form submit')
        setCustomer('')
    }

    // принимаем событие от кнопки "добавить"
    const handleSubmitCreate = async (e) => {
        e.preventDefault()
        console.info('Try to add item');

        const requestParams = {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            }
        };
        await fetch(`http://localhost:8080/order?customer=${customerName}`, requestParams)
            .then((data) => {
                getCustomerOrders(customerName)
                getAll('customer').then((data) => setCustomer(data))
            })
            .catch(function (error) {
                console.error('Error:', error);
                throw "Can't add order";
            });
    };

    function handleEdit(id) {
        console.info(`Start edit script`);

        Service.read(`order/${id}`)
            .then(function (data) {
                setCurrEditItem(data.id);
                setOrderSessions(data.sessions)
                setModalTable(true)
                console.info('End edit script');
            })
    };

    const handleSubmitAdd = async (e, id) => {
        console.info('Start add session');
        e.preventDefault(); // страница перестает перезагружаться
        const requestParams = {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        };
        const requestUrl = `http://localhost:8080/order/${id}?session=${sessionName}&count=${count}`
        const response = await fetch(requestUrl, requestParams)
        await response.json()
            .then((data) => {
                setOrderSessions(data.sessions)
                console.info('End add session');
            })
            .catch((error) => {
                console.error('Error:', error);
                alert('Не хватает билетов на сеанс')
            });
    };

    const handleSubmitDelete = async (e, id) => {
        console.info('Start delete session');
        e.preventDefault(); // страница перестает перезагружаться
        const requestParams = {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        };
        const requestUrl = `http://localhost:8080/order/${id}?session=${sessionName}&count=-${count}`
        const response = await fetch(requestUrl, requestParams)
        await response.json()
            .then((data) => {
                console.info('End delete session')
                setOrderSessions(data.sessions)
            })
            .catch((error) => {
                console.error('Error:', error);
            });
    };

    function handleDelete(id) {
        console.info('Try to remove item');
        Service.delete(`order/${id}`)
            .then(() => {
                setUsers(users.filter(elem => elem.id !== id))
                console.log("Removed")
            });
    };

    async function getCustomerOrders(id) {
        Service.read(`customer/${id}`)
            .then(function (data) {
                setUsers(data.orders);
                console.info('End');
            })
            .catch(e => { console.log('Error get orders') })
    }

    async function handleDeleteOrderSession(e, id, sessionId) {
        console.info('Start delete session');
        const requestParams = {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        };
        const requestUrl = `http://localhost:8080/order/${id}?session=${sessionId}`
        const response = await fetch(requestUrl, requestParams)
        await response.json()
            .then((data) => {
                console.info('End delete session')
                setOrderSessions(data.sessions)
            })
            .catch((error) => {
                console.error('Error:', error);
            });
    }

    const Content = (
        <>
            <select required className="form-select" name="customer" id="customer" value={customer.value} onChange={e => {
                setCustomerName(e.target.value)
                getCustomerOrders(e.target.value)
            }} >
                <option value='' defaultValue disabled>Выберите значение</option>
                {customer ? customer.map(elem =>
                    <option key={elem.id} value={elem.id}>{elem.login}</option>
                ) : null}
            </select>
            <form className="d-flex flex-column fs-4 fw-bold text-white text-center align-items-center">
                <div>
                    <button className="btn btn-success my-3" type="button" onClick={handleSubmit}>Добавить</button>
                </div>
            </form>
            <table className="table mt-3 text-white">
                <thead>
                    <tr>
                        <th scope="col">#</th>
                        <th scope="col">Customer</th>
                        <th scope="col">DateOfPurchase</th>
                        <th scope="col"></th>
                    </tr>
                </thead>
                <tbody id="tbody">
                    {users.map((user) => (
                        <OrderItem
                            item={user}
                            key={user.id}
                            editFunc={handleEdit}
                            removeFunc={handleDelete}
                        />
                    ))}
                </tbody>
            </table>
            <ModalEdit visible={modalTable} setVisible={setModalTable}>
                <form className="fs-4 description fw-bold d-flex flex-column align-items-center" id="frm-items-edit">
                    <div className="col-6">
                        <label className="form-label" htmlFor="session">Сеанс</label>
                        <select required className="form-control" name="session" id="session" value={session.value} onChange={e => setSessionName(e.target.value)} >
                            <option value='' defaultValue disabled>Выберите значение</option>
                            {session ? session.map(elem =>
                                <option key={elem.id} value={elem.id}>{elem.cinema.name} {new Date(elem.timestamp).toLocaleString('RU-ru')}</option>
                            ) : null}
                        </select>
                    </div>
                    <div className="col-6">
                        <label className="form-label" htmlFor="count">Количество</label>
                        <input required className="form-control" name="count" id="count" type="number" min="1" value={count} onChange={e => setCount(e.target.value)} placeholder="Введите количество" />
                    </div>
                    <div className="text-center mt-3">
                        <button className="btn btn-success mx-1" type="button" onClick={e => handleSubmitAdd(e, currEditItem)}>Добавить</button>
                        <button className="btn btn-danger mx-1" type="button" onClick={e => handleSubmitDelete(e, currEditItem)}>Удалить</button>
                        <button className="btn btn-secondary mx-1" type="button" data-bs-dismiss="modal" onClick={() => setModalTable(false)}>Отмена</button>
                    </div>
                    <p>Мои сеансы</p>
                    <table className="table fs-6 table-bordered" id="tbl-items">
                        <thead>
                            <tr>
                                <th scope="col">#</th>
                                <th scope="col">Price</th>
                                <th scope="col">Cinema</th>
                                <th scope="col">Timestamp</th>
                                <th scope="col">Count</th>
                            </tr>
                        </thead>
                        <tbody>
                            {orderSessions ? orderSessions.map((item) =>
                                <OrderSessionItem
                                    item={item}
                                    key={parseInt(item.sessionId+''+item.orderId)}
                                    removeFunc={e => handleDeleteOrderSession(e, currEditItem, item.sessionId)}
                                />
                            ) : null}
                        </tbody>
                    </table>
                </form>
            </ModalEdit>
        </>
    )

    return (
        <ContentBlock className="d-flex justify-content-center flex-wrap" valueBlock={Content} title='Заказы' />
    )
}
