import { React, useState, useEffect } from 'react'
import ContentBlock from './components/ContentBlock'
import Service from './services/Service';
import ModalEdit from './components/ModalEdit';
import MyButton from './components/MyButton';
import UserItem from './components/UserItem';

export default function Registration() {
    const [login, setLogin] = useState('');
    const [password, setPassword] = useState('');
    const [loginEdit, setLoginEdit] = useState('');
    const [passwordEdit, setPasswordEdit] = useState('');
    const [users, setUsers] = useState([]);
    const [error, setError] = useState(false);
    const [modalTable, setModalTable] = useState(false);
    // хук для запоминания индекса элемента, вызвавшего модальное окно
    const [currEditItem, setCurrEditItem] = useState(0);

    useEffect(() => {
        setError(false)
        getAll()
    }, [])

    async function getAll() {
        const requestUrl = "http://localhost:8080/customer"
        const response = await fetch(requestUrl)
        const temp = await response.json()
        setUsers(temp)
    }

    function handleSubmit(e) {
        e.preventDefault();
        if (login.length <= 0 || password.length <= 0) {
            setError(true)
            throw 'Form not submit'
        }
        handleSubmitCreate(e)
        console.log('Form submit')
        setError(false)
        setLogin('')
        setPassword('')
    }

    // принимаем событие от кнопки "добавить"
    const handleSubmitCreate = async (e) => {
        e.preventDefault()
        console.info('Try to add item');
        const requestParams = {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            }
        };
        await fetch(`http://localhost:8080/customer?login=${login}&password=${password}`, requestParams)
            .then(() => {
                getAll()
            })
    };

    function handleEdit(id) {
        console.info(`Start edit script`);

        Service.read(`customer/${id}`)
            .then(function (data) {
                setLoginEdit(data.login);
                setPasswordEdit(data.password);
                setCurrEditItem(data.id);
                setModalTable(true)
                console.info('End edit script');
            })
    };

    const handleSubmitEdit = async (e, id) => {
        console.info('Start synchronize edit');
        e.preventDefault(); // страница перестает перезагружаться
        const requestParams = {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        };
        const requestUrl = `http://localhost:8080/customer/${id}?login=${loginEdit}&password=${passwordEdit}`
        const response = await fetch(requestUrl, requestParams)
        const temp = await response.json()
            .then((data) => {
                setUsers(
                    users.map(item =>
                        item.id === id ? {
                            ...item,
                            login: data.login,
                            password: data.password
                        } : item)
                )
                console.info('End synchronize edit');
                setModalTable(false)
            })
            .catch((error) => {
                console.error('Error:', error);
            });
    };

    function handleDelete(id) {
        console.info('Try to remove item');
        Service.delete(`customer/${id}`)
            .then(() => {
                setUsers(users.filter(elem => elem.id !== id))
                console.log("Removed")
            });
    };

    const Content = (
        <>
            <form onSubmit={handleSubmit} className="d-flex flex-column fs-4 fw-bold text-white text-center align-items-center">
                <div>
                    <label className="form-label">Логин</label>
                    <input className="form-control mainInput" type="text" value={login} onChange={e => setLogin(e.target.value)} placeholder="Введите логин" />
                </div>
                {error && login.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод логина</label> : null}
                <div>
                    <label className="form-label">Пароль</label>
                    <input className="form-control mainInput" type="text" value={password} onChange={e => setPassword(e.target.value)} placeholder="Введите пароль" />
                </div>
                {error && password.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод пароля</label> : null}
                <div>
                    <button className="btn btn-success my-3" type="submit">Регистрация</button>
                </div>
            </form>
            <table className="table mt-3 text-white">
                <thead>
                    <tr>
                        <th scope="col">#</th>
                        <th scope="col">Login</th>
                        <th scope="col">Password</th>
                        <th scope="col"></th>
                    </tr>
                </thead>
                <tbody id="tbody">
                    {users.map((user) => (
                        <UserItem
                            item={user}
                            key={user.id}
                            editFunc={handleEdit}
                            removeFunc={handleDelete} />
                    ))}
                </tbody>
            </table>
            <ModalEdit visible={modalTable} setVisible={setModalTable}>
                <form className="g-3 fs-4 description fw-bold container" id="frm-items-edit" onSubmit={(e) => handleSubmitEdit(e, currEditItem)}>
                    <div className="row">
                        <label className="form-label" htmlFor="loginEdit">Логин</label>
                        <input value={loginEdit} onChange={e => setLoginEdit(e.target.value)} className="form-control" name='loginEdit' id="loginEdit" type="text" placeholder="Введите логин" required />
                    </div>
                    <div className="row">
                        <label className="form-label" htmlFor="passwordEdit">Пароль</label>
                        <input value={passwordEdit} onChange={e => setPasswordEdit(e.target.value)} className="form-control" name='passwordEdit' id="passwordEdit" type="text" placeholder="Введите пароль" required />
                    </div>
                    <div className="text-center mt-3">
                        <button className="btn btn-primary mx-1" type="submit" id="buttonSaveChanges">Сохранить изменения</button>
                        <button className="btn btn-secondary mx-1" type="button" data-bs-dismiss="modal" onClick={() => setModalTable(false)}>Отмена</button>
                    </div>
                </form>
            </ModalEdit>
        </>
    )

    return (
        <ContentBlock className="d-flex justify-content-center flex-wrap" valueBlock={Content} title='Регистрация' />
    )
}
