import { React, useState, useEffect } from 'react'
import ContentBlock from './components/ContentBlock'
import Service from './services/Service';
import ModalEdit from './components/ModalEdit';
import SessionItem from './components/SessionItem';
import CinemaDto from './models/CinemaDto'

export default function Sessions() {
    const [price, setPrice] = useState(1);
    const [timestamp, setTimestamp] = useState(new Date());
    const [maxCount, setMaxCount] = useState(1);
    const [priceEdit, setPriceEdit] = useState('');
    const [users, setUsers] = useState([]);
    const [error, setError] = useState(false);
    const [modalTable, setModalTable] = useState(false);
    // хук для запоминания индекса элемента, вызвавшего модальное окно
    const [currEditItem, setCurrEditItem] = useState(0);
    // для выпадающих значений
    const [cinemaName, setCinemaName] = useState('');
    const [cinema, setCinema] = useState([]);

    useEffect(() => {
        setError(false)
        getAll('cinema').then((data) => setCinema(data))
        getAll('session').then((data) => setUsers(data))
    }, [])

    async function getAll(elem) {
        const requestUrl = `http://localhost:8080/${elem}`
        const response = await fetch(requestUrl)
        const result = await response.json()
        return result
    }

    function handleSubmit(e) {
        e.preventDefault();
        if (price.length <= 0 || cinema.length <= 0) {
            setError(true)
            throw 'Form not submit'
        }
        handleSubmitCreate(e)
        console.log('Form submit')
        setError(false)
        setPrice('')
        setCinema('')
    }

    // принимаем событие от кнопки "добавить"
    const handleSubmitCreate = async (e) => {
        e.preventDefault()
        console.info('Try to add item');
        const requestParams = {
            method: "POST",
            headers: {
                "Content-Type": "application/json"
            }
        };
        await fetch(`http://localhost:8080/session?price=${price}&timestamp=${timestamp}&cinemaid=${cinemaName}&capacity=${maxCount}`, requestParams)
            .then(() => {
                getAll('session').then((data) => setUsers(data))
                getAll('cinema').then((data) => setCinema(data))
            })
            .catch(function (error) {
                console.error('Error:', error);
                throw "Can't add session";
            });
    };

    function handleEdit(id) {
        console.info(`Start edit script`);

        Service.read(`session/${id}`)
            .then(function (data) {
                setPriceEdit(data.price);
                setCurrEditItem(data.id);
                setModalTable(true)
                console.info('End edit script');
            })
    };

    const handleSubmitEdit = async (e, id) => {
        console.info('Start synchronize edit');
        e.preventDefault(); // страница перестает перезагружаться
        const requestParams = {
            method: "PUT",
            headers: {
                "Content-Type": "application/json"
            }
        };
        const requestUrl = `http://localhost:8080/session/${id}?price=${priceEdit}`
        const response = await fetch(requestUrl, requestParams)
        await response.json()
            .then((data) => {
                setUsers(
                    users.map(item =>
                        item.id === id ? {
                            ...item,
                            price: data.price
                        } : item)
                )
                console.info('End synchronize edit');
                setModalTable(false)
            })
            .catch((error) => {
                console.error('Error:', error);
            });
    };

    function handleDelete(id) {
        console.info('Try to remove item');
        Service.delete(`session/${id}`)
            .then(() => {
                setUsers(users.filter(elem => elem.id !== id))
                console.log("Removed")
            });
    };

    const Content = (
        <>
            <form className="d-flex flex-column fs-4 fw-bold text-white text-center align-items-center">
                <div>
                    <label className="form-label">Цена</label>
                    <input className="form-control mainInput" min="1" type="number" step="0.01" value={price} onChange={e => setPrice(e.target.value)} placeholder="Введите цену" />
                </div>
                {error && price.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод цены</label> : null}
                <div>
                    <label className="form-label">Фильм</label>
                    <select required className="form-select" name="filmCountry" id="filmCountry" value={cinema.value} onChange={e => setCinemaName(e.target.value)} >
                        <option value='' defaultValue disabled>Выберите значение</option>
                        {cinema ? cinema.map(elem =>
                            <option key={elem.id} value={elem.id}>{elem.name}</option>
                        ) : null }
                    </select>
                </div>
                {error && cinema.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод фильма</label> : null}
                <div>
                    <label className="form-label">Кол-во сеансов</label>
                    <input className="form-control mainInput" min="1" type="number" value={maxCount} onChange={e => setMaxCount(e.target.value)} placeholder="Введите количество" />
                </div>
                {error && price.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод цены</label> : null}
                <div>
                    <label className="form-label">Дата</label>
                    <input className="form-control mainInput" type="datetime-local" value={timestamp} onChange={e => setTimestamp(e.target.value)} placeholder="Введите дату" />
                </div>
                {error && timestamp.length <= 0 ? <label className="fs-6 text-danger">Неправильный ввод даты</label> : null}
                <div>
                    <button className="btn btn-success my-3" type="button" onClick={handleSubmit}>Сохранить</button>
                </div>
            </form>
            <table className="table mt-3 text-white">
                <thead>
                    <tr>
                        <th scope="col">#</th>
                        <th scope="col">Price</th>
                        <th scope="col">Cinema</th>
                        <th scope="col">Timestamp</th>
                        <th scope="col">Capacity</th>
                        <th scope="col">MaxCount</th>
                        <th scope="col"></th>
                    </tr>
                </thead>
                <tbody id="tbody">
                    {users.map((user) => (
                        <SessionItem
                            item={user}
                            key={user.id}
                            editFunc={handleEdit}
                            removeFunc={handleDelete}
                            />
                    ))}
                </tbody>
            </table>
            <ModalEdit visible={modalTable} setVisible={setModalTable}>
                <form className="g-3 fs-4 description fw-bold container" id="frm-items-edit" onSubmit={(e) => handleSubmitEdit(e, currEditItem)}>
                    <div className="row">
                        <label className="form-label" htmlFor="priceEdit">Цена</label>
                        <input value={priceEdit} onChange={e => setPriceEdit(e.target.value)} className="form-control" name='priceEdit' id="priceEdit" type="number" step="0.01" min="1" placeholder="Введите цену" required />
                    </div>
                    <div className="text-center mt-3">
                        <button className="btn btn-primary mx-1" type="submit" id="buttonSaveChanges">Сохранить изменения</button>
                        <button className="btn btn-secondary mx-1" type="button" data-bs-dismiss="modal" onClick={() => setModalTable(false)}>Отмена</button>
                    </div>
                </form>
            </ModalEdit>
        </>
    )

    return (
        <ContentBlock className="d-flex justify-content-center flex-wrap" valueBlock={Content} title='Сеансы' />
    )
}
