package com.labwork1.app.student.controller;

import com.labwork1.app.configuration.WebConfiguration;
import com.labwork1.app.student.service.CinemaService;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/cinema")
public class CinemaController {
    private final CinemaService cinemaService;

    public CinemaController(CinemaService cinemaService) {
        this.cinemaService = cinemaService;
    }

    @GetMapping("/{id}")
    public CinemaDto getCinema(@PathVariable Long id) {
        return new CinemaDto(cinemaService.findCinema(id));
    }

    @GetMapping
    public List<CinemaDto> getCinemas() {
        return cinemaService.findAllCinemas().stream()
                .map(CinemaDto::new)
                .toList();
    }

    @GetMapping("/search")
    public List<CinemaDto> getCinemas(@RequestParam("request") String request) {
        return cinemaService.findAllCinemas(request).stream()
                .map(CinemaDto::new)
                .toList();
    }

    @PostMapping
    public CinemaDto createCinema(@RequestBody @Valid CinemaDto cinemaDto) {
        return new CinemaDto(cinemaService.addCinema(cinemaDto));
    }

    @PutMapping("/{id}")
    public CinemaDto updateCinema(@RequestBody @Valid CinemaDto cinemaDto) {
        return new CinemaDto(cinemaService.updateCinema(cinemaDto));
    }

    @DeleteMapping("/{id}")
    public CinemaDto deleteCinema(@PathVariable Long id) {
        return new CinemaDto(cinemaService.deleteCinema(id));
    }

}
