package com.labwork1.app.student.controller;

import com.labwork1.app.student.model.UserRole;
import com.labwork1.app.student.service.CinemaService;
import jakarta.validation.Valid;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Base64;
import java.util.List;

@Controller
@RequestMapping("/cinema")
public class CinemaMvcController {
    private final CinemaService cinemaService;

    public CinemaMvcController(CinemaService cinemaService) {
        this.cinemaService = cinemaService;
    }

    @GetMapping
    public String getCinemas(Model model) {
        model.addAttribute("cinemas",
                cinemaService.findAllCinemas().stream()
                        .map(CinemaDto::new)
                        .toList());
        return "cinema";
    }

    @GetMapping(value = {"/edit", "/edit/{id}"})
    @Secured({UserRole.AsString.ADMIN})
    public String editCinema(@PathVariable(required = false) Long id,
                             Model model) {
        if (id == null || id <= 0) {
            model.addAttribute("cinemaDto", new CinemaDto());
        } else {
            model.addAttribute("cinemaId", id);
            model.addAttribute("cinemaDto", new CinemaDto(cinemaService.findCinema(id)));
        }
        return "cinema-edit";
    }

    @GetMapping(value = "/search/")
    public String searchCinema(@RequestParam String request,
                               Model model) {
        List<CinemaDto> cinemas = cinemaService.findAllCinemas(request)
                .stream().map(CinemaDto::new).toList();
        model.addAttribute("cinemas", cinemas);
        return "cinema";
    }

    @PostMapping(value = {"/", "/{id}"})
    @Secured({UserRole.AsString.ADMIN})
    public String saveCinema(@PathVariable(required = false) Long id,
                             @RequestParam("multipartFile") MultipartFile multipartFile,
                             @ModelAttribute @Valid CinemaDto cinemaDto,
                             BindingResult bindingResult,
                             Model model) throws IOException {
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "cinema-edit";
        }
        if (id == null || id <= 0) {
            cinemaDto.setImage("data:" + multipartFile.getContentType() + ";base64," + Base64.getEncoder().encodeToString(multipartFile.getBytes()));
            cinemaService.addCinema(cinemaDto);
        } else {
            cinemaDto.setId(id);
            cinemaDto.setImage("data:" + multipartFile.getContentType() + ";base64," + Base64.getEncoder().encodeToString(multipartFile.getBytes()));
            cinemaService.updateCinema(cinemaDto);
        }
        return "redirect:/cinema";
    }

    @PostMapping("/delete/{id}")
    @Secured({UserRole.AsString.ADMIN})
    public String deleteCinema(@PathVariable Long id) {
        cinemaService.deleteCinema(id);
        return "redirect:/cinema";
    }

}
