package com.labwork1.app.student.controller;

import com.labwork1.app.configuration.WebConfiguration;
import com.labwork1.app.student.model.UserSignupDto;
import com.labwork1.app.student.service.CustomerService;
import jakarta.validation.Valid;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/customer")
public class CustomerController {
    private final CustomerService customerService;

    public CustomerController(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping("/{id}")
    public CustomerDto getCustomer(@PathVariable Long id) {
        return new CustomerDto(customerService.findCustomer(id));
    }

    @GetMapping
    public List<CustomerDto> getCustomers() {
        return customerService.findAllCustomers().stream()
                .map(CustomerDto::new)
                .toList();
    }

    @PostMapping
    public CustomerDto createCustomer(@RequestBody @Valid UserSignupDto userSignupDto) {
        return new CustomerDto(customerService.addCustomer(userSignupDto));
    }

    @PutMapping("/{id}")
    public CustomerDto updateCustomer(@PathVariable Long id,
                                      @RequestParam("login") String login,
                                      @RequestParam("password") String password) {
        return new CustomerDto(customerService.updateCustomer(id, login, password));
    }

    @DeleteMapping("/{id}")
    public CustomerDto deleteCustomer(@PathVariable Long id) {
        return new CustomerDto(customerService.deleteCustomer(id));
    }
}
