package com.labwork1.app.student.controller;

import com.labwork1.app.student.model.UserRole;
import com.labwork1.app.student.service.CustomerService;
import jakarta.validation.Valid;
import org.springframework.data.domain.Page;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.stream.IntStream;

@Controller
@RequestMapping("/customer")
@Secured({UserRole.AsString.ADMIN})
public class CustomerMvcController {
    private final CustomerService customerService;

    public CustomerMvcController(CustomerService customerService) {
        this.customerService = customerService;
    }

    @GetMapping
    public String getCustomers(@RequestParam(defaultValue = "1") int page,
                               @RequestParam(defaultValue = "5") int size,
                               Model model) {
        final Page<CustomerDto> users = customerService.findAllPages(page, size)
                .map(CustomerDto::new);
        model.addAttribute("customers", users);
        final int totalPages = users.getTotalPages();
        final List<Integer> pageNumbers = IntStream.rangeClosed(1, totalPages)
                .boxed()
                .toList();
        model.addAttribute("pages", pageNumbers);
        model.addAttribute("totalPages", totalPages);
        return "customer";
    }

    @GetMapping(value = {"/edit", "/edit/{id}"})
    public String editCustomer(@PathVariable(required = false) Long id,
                               Model model) {
        if (id == null || id <= 0) {
            model.addAttribute("customerDto", new CustomerDto());
        } else {
            model.addAttribute("customerId", id);
            model.addAttribute("customerDto", new CustomerDto(customerService.findCustomer(id)));
        }
        return "customer-edit";
    }

    @PostMapping("/delete/{id}")
    public String deleteCustomer(@PathVariable Long id) {
        customerService.deleteCustomer(id);
        return "redirect:/customer";
    }
}
