package com.labwork1.app.student.controller;

import com.labwork1.app.configuration.WebConfiguration;
import com.labwork1.app.student.service.OrderService;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/order")
public class OrderController {
    private final OrderService orderService;

    public OrderController(OrderService orderService) {
        this.orderService = orderService;
    }

    @GetMapping("/{id}")
    public OrderDto getOrder(@PathVariable Long id) {
        return new OrderDto(orderService.findOrder(id));
    }

    @GetMapping
    public List<OrderDto> getOrders() {
        return orderService.findAllOrders().stream()
                .map(OrderDto::new)
                .toList();
    }

    @PostMapping
    public OrderDto createOrder(@RequestParam("customer") Long customer) {
        return new OrderDto(orderService.addOrder(customer));
    }

    @PutMapping("/{id}")
    public OrderDto updateOrder(@PathVariable Long id,
                                @RequestParam("session") Long session,
                                @RequestParam(value = "count", required = false) Integer count) {
        if (count == null)
            return new OrderDto(orderService.deleteSessionInOrder(id, session, Integer.MAX_VALUE));
        if (count > 0)
            return new OrderDto(orderService.addSession(id, session, count));
        return new OrderDto(orderService.deleteSessionInOrder(id, session, -count));
    }

    @DeleteMapping("/{id}")
    public OrderDto deleteOrder(@PathVariable Long id) {
        return new OrderDto(orderService.deleteOrder(id));
    }

}
