package com.labwork1.app.student.controller;

import com.labwork1.app.configuration.WebConfiguration;
import com.labwork1.app.student.service.SessionService;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping(WebConfiguration.REST_API + "/session")
public class SessionController {
    private final SessionService sessionService;

    public SessionController(SessionService sessionService) {
        this.sessionService = sessionService;
    }

    @GetMapping("/{id}")
    public SessionDto getSession(@PathVariable Long id) {
        return new SessionDto(sessionService.findSession(id));
    }

    @GetMapping
    public List<SessionDto> getSessions() {
        return sessionService.findAllSessions().stream()
                .map(SessionDto::new)
                .toList();
    }

    @PostMapping
    public SessionDto createSession(@RequestParam("price") String price,
                                    @RequestParam("timestamp") String timestamp,
                                    @RequestParam("cinemaid") Long cinemaId,
                                    @RequestParam("capacity") Integer capacity) throws ParseException {
        SimpleDateFormat format = new SimpleDateFormat();
        format.applyPattern("yyyy-MM-dd-HH:mm");
        Date docDate = format.parse(timestamp.replace('T', '-'));
        return new SessionDto(sessionService.findSession(
                sessionService.addSession(Double.parseDouble(price),
                        LocalDateTime.now(), cinemaId, capacity).getId()));
    }

    @PutMapping("/{id}")
    public SessionDto updateSession(@PathVariable Long id,
                                    @RequestParam("price") String price) {
        return new SessionDto(sessionService.findSession(sessionService
                .updateSession(id, Double.parseDouble(price)).getId()));
    }

    @DeleteMapping("/{id}")
    public SessionDto deleteSession(@PathVariable Long id) {
        return new SessionDto(sessionService.deleteSession(id));
    }

}
