package com.labwork1.app.student.controller;

import com.labwork1.app.student.model.UserRole;
import com.labwork1.app.student.service.CinemaService;
import com.labwork1.app.student.service.SessionService;
import jakarta.validation.Valid;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;

@Controller
@RequestMapping("/session")
public class SessionMvcController {
    private final SessionService sessionService;
    private final CinemaService cinemaService;

    public SessionMvcController(SessionService sessionService, CinemaService cinemaService) {
        this.sessionService = sessionService;
        this.cinemaService = cinemaService;
    }

    @GetMapping
    public String getSessions(Model model) {
        model.addAttribute("sessions", sessionService.findAllSessions().stream()
                .map(SessionDto::new)
                .toList());
        return "session";
    }

    @GetMapping(value = {"/edit", "/edit/{id}"})
    @Secured({UserRole.AsString.ADMIN})
    public String editSession(@PathVariable(required = false) Long id,
                              Model model) {
        if (id == null || id <= 0) {
            List<CinemaDto> cinemas = cinemaService.findAllCinemas().stream()
                    .map(CinemaDto::new)
                    .toList();
            model.addAttribute("sessionDto", new SessionDto());
            model.addAttribute("cinemas", cinemas);
        } else {
            model.addAttribute("sessionId", id);
            model.addAttribute("sessionDto", new SessionDto(sessionService.findSession(id)));
        }
        return "session-edit";
    }

    @PostMapping(value = "/{id}")
    @Secured({UserRole.AsString.ADMIN})
    public String editSession(@PathVariable Long id,
                              @ModelAttribute @Valid SessionDto sessionDto,
                              BindingResult bindingResult,
                              Model model) {
        if (bindingResult.hasErrors()) {
            model.addAttribute("errors", bindingResult.getAllErrors());
            return "session-edit";
        }
        sessionService.updateSession(id, sessionDto.getPrice());

        return "redirect:/session";
    }

    @PostMapping(value = "/")
    @Secured({UserRole.AsString.ADMIN})
    public String saveSession(@RequestParam("price") String price,
                              @RequestParam("timestamp") LocalDateTime timestamp,
                              @RequestParam("cinemaid") Long cinemaId,
                              @RequestParam("maxCount") Integer capacity,
                              Model model) {
        sessionService.addSession(Double.parseDouble(price), timestamp,
                cinemaId, capacity);
        return "redirect:/session";
    }

    @PostMapping("/delete/{id}")
    @Secured({UserRole.AsString.ADMIN})
    public String deleteSession(@PathVariable Long id) {
        sessionService.deleteSession(id);
        return "redirect:/session";
    }

}
