package com.labwork1.app.student.model;

import com.labwork1.app.student.controller.CinemaDto;
import jakarta.persistence.*;
import jakarta.validation.constraints.NotBlank;

import java.util.ArrayList;
import java.util.List;

@Entity
public class Cinema {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    @NotBlank(message = "name can't be null or empty")
    @Column
    private String name;
    @OneToMany(fetch = FetchType.EAGER, mappedBy = "cinema", cascade = CascadeType.REMOVE)
    private List<Session> sessions;
    @Lob
    private byte[] image;

    public Cinema() {
    }

    public Cinema(String name) {
        this.name = name;
        this.sessions = new ArrayList<>();
    }

    public Cinema(CinemaDto cinemaDto) {
        this.name = cinemaDto.getName();
        this.image = cinemaDto.getImage().getBytes();
        this.sessions = new ArrayList<>();
    }

    public byte[] getImage() {
        return image;
    }

    public void setImage(byte[] image) {
        this.image = image;
    }

    public List<Session> getSessions() {
        return sessions;
    }

    public void setSession(Session session) {
        if (session.getCinema().equals(this)) {
            this.sessions.add(session);
        }
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }
}
