package com.labwork1.app.student.model;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
@Table(name = "tab_order")
public class Order {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    @NotNull(message = "dateOfPurchase can't be null or empty")
    @Temporal(TemporalType.DATE)
    private Date dateOfPurchase;
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "customer_fk")
    private Customer customer;
    @OneToMany(mappedBy = "order", fetch = FetchType.EAGER, cascade =
            {
                    CascadeType.REMOVE,
                    CascadeType.MERGE,
                    CascadeType.PERSIST
            }, orphanRemoval = true)
    private List<OrderSession> sessions;

    public Order() {
    }

    public Order(Date dateOfPurchase) {
        this.dateOfPurchase = dateOfPurchase;
    }

    public void addSession(OrderSession orderSession) {
        if (sessions == null) {
            sessions = new ArrayList<>();
        }
        if (!sessions.contains(orderSession))
            this.sessions.add(orderSession);
    }

    public void removeSession(OrderSession orderSession){
        if (sessions.contains(orderSession))
            this.sessions.remove(orderSession);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Order order = (Order) o;
        return Objects.equals(id, order.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    public String toString() {
        return "Order {" +
                "id=" + id +
                ", date='" + dateOfPurchase.toString() + '\'' +
                ", customer='" + customer.toString() + '\'';
    }

    public Long getId() {
        return id;
    }

    public Date getDateOfPurchase() {
        return dateOfPurchase;
    }

    public void setDateOfPurchase(Date dateOfPurchase) {
        this.dateOfPurchase = dateOfPurchase;
    }

    public Customer getCustomer() {
        return customer;
    }

    public void setCustomer(Customer customer) {
        this.customer = customer;
        if (!customer.getOrders().contains(this)) {
            customer.setOrder(this);
        }
    }

    public List<OrderSession> getSessions() {
        return sessions;
    }

    public void setSessions(List<OrderSession> sessions) {
        this.sessions = sessions;
    }
}
