package com.labwork1.app.student.model;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotNull;
import org.springframework.format.annotation.DateTimeFormat;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

@Entity
public class Session {
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    protected Long id;
    @NotNull(message = "price can't be null or empty")
    protected Double price;
    @NotNull(message = "timestamp can't be null or empty")
    @Column
    @Temporal(TemporalType.TIMESTAMP)
    @DateTimeFormat(pattern = "yyyy-MM-dd'T'HH:mm")
    protected LocalDateTime timestamp;
    @OneToMany(mappedBy = "session", fetch = FetchType.EAGER)
    private List<OrderSession> orders;
    @ManyToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "cinema_fk")
    protected Cinema cinema;
    @NotNull(message = "maxCount can't be null or empty")
    @Column
    protected Integer maxCount;

    public Session() {
    }

    public Integer getMaxCount() {
        return maxCount;
    }

    public Session(Double price, LocalDateTime timestamp, Integer maxCount) {
        this.price = price;
        this.timestamp = timestamp;
        this.maxCount = maxCount;
    }

    public Session(Session session) {
        this.id = session.getId();
        this.price = session.getPrice();
        this.timestamp = session.getTimestamp();
        this.orders = session.getOrders();
        this.cinema = session.getCinema();
        this.maxCount = session.getMaxCount();
    }

    public Cinema getCinema() {
        return cinema;
    }
    public void setCinema(Cinema cinema) {
        this.cinema = cinema;
        if (!cinema.getSessions().contains(this)) {
            cinema.setSession(this);
        }
    }
    public void addOrder(OrderSession orderSession){
        if (orders == null){
            orders = new ArrayList<>();
        }
        if (!orders.contains(orderSession)) {
            this.orders.add(orderSession);
        }
    }

    public void removeOrder(OrderSession orderSession){
        if (orders.contains(orderSession))
            this.orders.remove(orderSession);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Session session = (Session) o;
        return Objects.equals(id, session.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

    @Override
    public String toString() {
        return "Session {" +
                "id=" + id +
                ", price='" + price + '\'' +
                ", timestamp='" + timestamp.toString() + '\'' +
                ", maxCount='" + maxCount.toString() + '\'' +
                ", cinema='" + cinema.toString() + '\'' +
                '}';
    }

    public Long getId() {
        return id;
    }

    public Double getPrice() {
        return price;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public void setOrders(List<OrderSession> orders) {
        this.orders = orders;
    }

    public void setMaxCount(Integer maxCount) {
        this.maxCount = maxCount;
    }

    public void setPrice(Double price) {
        this.price = price;
    }

    public LocalDateTime getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(LocalDateTime timestamp) {
        this.timestamp = timestamp;
    }

    public List<OrderSession> getOrders() {
        return orders;
    }
}
