package com.labwork1.app.student.service;

import com.labwork1.app.student.controller.CinemaDto;
import com.labwork1.app.student.model.Cinema;
import com.labwork1.app.student.repository.CinemaRepository;
import com.labwork1.app.util.validation.ValidatorUtil;
import jakarta.transaction.Transactional;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class CinemaService {
    private final CinemaRepository cinemaRepository;
    private final ValidatorUtil validatorUtil;

    public CinemaService(CinemaRepository cinemaRepository, ValidatorUtil validatorUtil) {
        this.cinemaRepository = cinemaRepository;
        this.validatorUtil = validatorUtil;
    }

    @Transactional
    public Cinema addCinema(CinemaDto dto) {
        final Cinema cinema = new Cinema(dto);
        validatorUtil.validate(cinema);
        return cinemaRepository.save(cinema);
    }

    @Transactional
    public Cinema addCinema(String name) {
        final Cinema cinema = new Cinema(name);
        validatorUtil.validate(cinema);
        return cinemaRepository.save(cinema);
    }

    @Transactional
    public Cinema findCinema(Long id) {
        final Optional<Cinema> cinema = cinemaRepository.findById(id);
        return cinema.orElseThrow(() -> new CinemaNotFoundException(id));
    }

    @Transactional
    public List<Cinema> findAllCinemas(String search) {
        return cinemaRepository.findAll(search);
    }

    @Transactional
    public List<Cinema> findAllCinemas() {
        return cinemaRepository.findAll();
    }

    @Transactional
    public Cinema updateCinema(CinemaDto cinema) {
        final Cinema currentCinema = findCinema(cinema.getId());
        currentCinema.setName(cinema.getName());
        currentCinema.setImage(cinema.getImage().getBytes());
        validatorUtil.validate(currentCinema);
        return cinemaRepository.save(currentCinema);
    }

    @Transactional
    public Cinema deleteCinema(Long id) {
        final Cinema currentCinema = findCinema(id);
        cinemaRepository.delete(currentCinema);
        return currentCinema;
    }

    @Transactional
    public void deleteAllCinemas() {
        cinemaRepository.deleteAll();
    }
}
